package ru.dbotthepony.mc.otm.menu;

import net.minecraft.world.Container;
import net.minecraft.world.SimpleContainer;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.inventory.MenuType;
import net.minecraft.world.item.ItemStack;
import net.minecraftforge.common.util.LazyOptional;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.block.entity.BlockEntityMatteryPoweredMachine;
import ru.dbotthepony.mc.otm.capability.IMatteryEnergyStorage;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.menu.data.BigDecimalDataContainer;
import ru.dbotthepony.mc.otm.menu.slot.BatterySlot;
import ru.dbotthepony.mc.otm.menu.widget.BatteryLevelWidget;

import javax.annotation.Nullable;

abstract public class PoweredMatteryMenu extends MatteryMenu {
	protected BlockEntityMatteryPoweredMachine tile;

	public BigDecimalDataContainer energy;
	public BigDecimalDataContainer max_energy;

	public BatteryLevelWidget<PoweredMatteryMenu> battery_widget;

	protected PoweredMatteryMenu(
			@Nullable MenuType<?> menuType,
			int containerID,
			Inventory inventory,
			@Nullable BlockEntityMatteryPoweredMachine tile
	) {
		super(menuType, containerID, inventory, tile);
		this.tile = tile;

		if (tile == null || tile.getCapability(MatteryCapability.ENERGY).resolve().isEmpty()) {
			energy = new BigDecimalDataContainer();
			max_energy = new BigDecimalDataContainer();

			battery_widget = new BatteryLevelWidget<>(this, 13, 14);
		} else {
			energy = new BigDecimalDataContainer() {
				@Override
				protected void updateValue() {
					LazyOptional<IMatteryEnergyStorage> capability = tile.getCapability(MatteryCapability.ENERGY);

					if (capability.isPresent())
						setDecimal(capability.resolve().get().getBatteryLevel());
				}
			};

			max_energy = new BigDecimalDataContainer() {
				@Override
				protected void updateValue() {
					LazyOptional<IMatteryEnergyStorage> capability = tile.getCapability(MatteryCapability.ENERGY);

					if (capability.isPresent())
						setDecimal(capability.resolve().get().getMaxBatteryLevel());
				}
			};

			battery_widget = new BatteryLevelWidget<>(this, 13, 14, tile.getCapability(MatteryCapability.ENERGY).resolve().get());
		}

		this.addDataSlots(energy);
		this.addDataSlots(max_energy);
	}

	protected void addBatterySlot() {
		addBatterySlot(8, 66);
	}

	protected void addBatterySlot(int x) {
		addBatterySlot(x, 66);
	}

	protected void addBatterySlot(int x, int y) {
		Container battery_container = tile != null ? tile.battery_container : new SimpleContainer(1);
		this.addSlot(new BatterySlot(battery_container, 0, x, y, true));
	}
}
