package ru.dbotthepony.mc.otm.menu.widget;

import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.FormattedText;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.world.inventory.ContainerData;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.capability.IMatteryEnergyStorage;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.menu.FormattingHelper;
import ru.dbotthepony.mc.otm.menu.MatteryMenu;
import ru.dbotthepony.mc.otm.menu.data.BigDecimalDataContainer;
import ru.dbotthepony.mc.otm.screen.MatteryScreen;

import java.math.BigDecimal;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.function.Consumer;
import java.util.function.Supplier;

public class BatteryLevelWidget<T extends MatteryMenu> extends VerticalGaugeWidget<T> {
	protected Supplier<BigDecimal> value_supplier;
	protected Supplier<BigDecimal> max_value_supplier;

	protected final BigDecimalDataContainer value_container = new BigDecimalDataContainer();
	protected final BigDecimalDataContainer max_value_container = new BigDecimalDataContainer();

	public BatteryLevelWidget(T menu, int x, int y, IMatteryEnergyStorage capability) {
		this(menu, x, y, capability::getBatteryLevel, capability::getMaxBatteryLevel);
	}

	public BatteryLevelWidget(T menu, int x, int y) {
		super(menu, x, y);
		addDataSlots(value_container);
		addDataSlots(max_value_container);
	}

	public BatteryLevelWidget(T menu, int x, int y, Supplier<BigDecimal> value_supplier, Supplier<BigDecimal> max_value_supplier) {
		this(menu, x, y);
		this.value_supplier = value_supplier;
		this.max_value_supplier = max_value_supplier;
	}

	@Override
	public void updateServer() {
		if (value_supplier == null || max_value_supplier == null)
			return;

		value_container.setDecimal(value_supplier.get());
		max_value_container.setDecimal(max_value_supplier.get());
	}

	@Override
	public int getImageWidth() {
		return 9;
	}

	@Override
	public int getImageHeight() {
		return 48;
	}

	@Override
	public int getImageX() {
		return 0;
	}

	@Override
	public int getImageY() {
		return 48;
	}

	@Override
	public boolean renderTooltip(MatteryScreen<T> screen, PoseStack pose, int local_x, int local_y, int mouse_x, int mouse_y) {
		if (is_hovered) {
			List<Component> text = List.of(
				new TranslatableComponent("otm.gui.power.percentage_level", String.format("%.2f", getLevel() * 100d)),
				FormattingHelper.formatPowerLevel(value_container.getDecimal(), max_value_container.getDecimal())
			);

			screen.renderComponentTooltip(pose, text, mouse_x, mouse_y);

			return true;
		}

		return false;
	}

	@Override
	float getLevel() {
		return max_value_container.getDecimal().compareTo(BigDecimal.ZERO) == 0 ? 0f : value_container.getDecimal().divide(max_value_container.getDecimal(), MatteryCapability.ROUND_RULES).floatValue();
	}
}
