package ru.dbotthepony.mc.otm.screen;

import com.mojang.blaze3d.systems.RenderSystem;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.client.gui.screens.inventory.AbstractContainerScreen;
import net.minecraft.client.renderer.GameRenderer;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.FormattedText;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.inventory.Slot;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.menu.FormattingHelper;
import ru.dbotthepony.mc.otm.menu.MatteryMenu;
import ru.dbotthepony.mc.otm.menu.slot.BatterySlot;
import ru.dbotthepony.mc.otm.menu.slot.MachineInputSlot;
import ru.dbotthepony.mc.otm.menu.slot.MachineOutputSlot;
import ru.dbotthepony.mc.otm.menu.widget.AbstractWidget;

import java.math.BigDecimal;
import java.util.List;
import java.util.function.Supplier;

public class MatteryScreen<T extends MatteryMenu> extends AbstractContainerScreen<T> {
	protected static final ResourceLocation CONTAINER_BASE = new ResourceLocation(OverdriveThatMatters.MOD_ID, "textures/gui/generic_machine.png");
	public static final ResourceLocation WIDGETS = new ResourceLocation(OverdriveThatMatters.MOD_ID, "textures/gui/widgets.png");

	protected ResourceLocation CONTAINER_BACKGROUND() {
		return CONTAINER_BASE;
	}

	protected Supplier<BigDecimal> power_supplier;
	protected Supplier<BigDecimal> max_power_supplier;

	protected int power_gauge_x = 13;
	protected int power_gauge_y = 14;

	protected Supplier<BigDecimal> matter_supplier;
	protected Supplier<BigDecimal> max_matter_supplier;

	protected Supplier<Float> progress_supplier;

	protected int matter_gauge_x = 22;
	protected int matter_gauge_y = 14;

	protected boolean hovering_power_gauge = false;
	protected boolean hovering_matter_gauge = false;

	protected int render_x;
	protected int render_y;

	public MatteryScreen(T p_97741_, Inventory p_97742_, Component p_97743_) {
		super(p_97741_, p_97742_, p_97743_);

		imageWidth = 176;
		imageHeight = 178;
		titleLabelY = 4;

		inventoryLabelY = imageHeight - 92;
	}

	@Override
	protected void renderTooltip(PoseStack pose, int mouseX, int mouseY) {
		super.renderTooltip(pose, mouseX, mouseY);

		if (this.hoveredSlot == null && menu.getCarried().isEmpty()) {
			for (AbstractWidget widget : menu.widget_list) {
				if (widget.renderTooltip(this, pose, render_x, render_y, mouseX, mouseY)) {
					return;
				}
			}

			if (hovering_power_gauge) {
				List<FormattedText> text = getFormattedPowerLevel();

				if (text.size() != 0)
					this.renderComponentToolTip(pose, text, mouseX, mouseY, font);
			} else if (hovering_matter_gauge) {
				List<FormattedText> text = getFormattedMatterLevel();

				if (text.size() != 0)
					this.renderComponentToolTip(pose, text, mouseX, mouseY, font);
			}
		}
	}

	protected List<FormattedText> getFormattedPowerLevel() {
		if (power_supplier != null && max_power_supplier != null)
			return getFormattedPowerLevel(power_supplier.get(), max_power_supplier.get());

		return List.of();
	}

	protected List<FormattedText> getFormattedMatterLevel() {
		if (matter_supplier != null && max_matter_supplier != null)
			return getFormattedMatterLevel(matter_supplier.get(), max_matter_supplier.get());

		return List.of();
	}

	protected List<FormattedText> getFormattedPowerLevel(BigDecimal current, BigDecimal max) {
		if (current == null || max == null)
			return List.of();

		return List.of(
			new TranslatableComponent("otm.gui.power.percentage_level", String.format("%.2f", level(current, max) * 100d)),
			FormattingHelper.formatPowerLevel(current, max)
		);
	}

	protected List<FormattedText> getFormattedMatterLevel(BigDecimal current, BigDecimal max) {
		if (current == null || max == null)
			return List.of();

		return List.of(
			new TranslatableComponent("otm.gui.matter.percentage_level", String.format("%.2f", level(current, max) * 100d)),
			FormattingHelper.formatMatterLevel(current, max)
		);
	}

	protected void renderVerticalGauge(PoseStack pose, int x, int y, float level, int image_x, int image_y, int width, int max_height) {
		RenderSystem.setShader(GameRenderer::getPositionTexShader);
		RenderSystem.setShaderColor(1.0F, 1.0F, 1.0F, 1.0F);
		RenderSystem.setShaderTexture(0, WIDGETS);

		this.blit(pose, render_x + x, render_y + y, image_x, image_y, width, max_height);

		if (level > 0.01f) {
			int intHeight = (int) Math.floor(level * (float) max_height + 0.5f);
			this.blit(pose, render_x + x, render_y + y + max_height - intHeight, image_x + width, image_y + max_height - intHeight, width, intHeight);
		}
	}

	protected boolean renderVerticalGauge(PoseStack pose, int x, int y, int mouseX, int mouseY, float level, int image_x, int image_y, int width, int max_height) {
		renderVerticalGauge(pose, x, y, level, image_x, image_y, width, max_height);

		x += leftPos;
		y += topPos;
		return x <= mouseX && x + 7 >= mouseX && y <= mouseY && y + 46 >= mouseY;
	}

	protected void renderHorizontalGauge(PoseStack pose, int x, int y, float level, int image_x, int image_y, int max_width, int height) {
		RenderSystem.setShader(GameRenderer::getPositionTexShader);
		RenderSystem.setShaderColor(1.0F, 1.0F, 1.0F, 1.0F);
		RenderSystem.setShaderTexture(0, WIDGETS);

		this.blit(pose, render_x + x, render_y + y, image_x, image_y, max_width, height);

		if (level > 0.01f) {
			int intWidth = (int) Math.floor(level * (float) max_width + 0.5f);
			this.blit(pose, render_x + x, render_y + y, image_x + max_width, image_y, intWidth, height);
		}
	}

	protected boolean renderHorizontalGauge(PoseStack pose, int x, int y, int mouseX, int mouseY, float level, int image_x, int image_y, int width, int max_height) {
		renderVerticalGauge(pose, x, y, level, image_x, image_y, width, max_height);

		x += leftPos;
		y += topPos;
		return x <= mouseX && x + 7 >= mouseX && y <= mouseY && y + 46 >= mouseY;
	}

	public static float level(BigDecimal a, BigDecimal b, boolean exact) {
		float level;

		if (b.compareTo(BigDecimal.ZERO) == 0) {
			level = 0f;
		} else {
			level = a.divide(b, MatteryCapability.ROUND_RULES).floatValue();

			if (!exact && level >= 0.98)
				level = 1f;
		}

		return level;
	}

	public static float level(BigDecimal a, BigDecimal b) {
		return level(a, b, true);
	}

	protected boolean renderMatterGauge(PoseStack pose, int mouseX, int mouseY) {
		if (matter_supplier != null && max_matter_supplier != null)
			return renderMatterGauge(pose, mouseX, mouseY, matter_supplier.get(), max_matter_supplier.get());

		return false;
	}

	protected boolean renderPowerGauge(PoseStack pose, int mouseX, int mouseY) {
		if (power_supplier != null && max_power_supplier != null)
			return renderPowerGauge(pose, mouseX, mouseY, power_supplier.get(), max_power_supplier.get());

		return false;
	}

	protected boolean renderMatterGauge(PoseStack pose, int mouseX, int mouseY, BigDecimal current_value, BigDecimal max_value) {
		return renderMatterGauge(pose, matter_gauge_x, matter_gauge_y, mouseX, mouseY, current_value, max_value);
	}

	protected boolean renderPowerGauge(PoseStack pose, int mouseX, int mouseY, BigDecimal current_value, BigDecimal max_value) {
		return renderPowerGauge(pose, power_gauge_x, power_gauge_y, mouseX, mouseY, current_value, max_value);
	}

	protected boolean renderPowerGauge(PoseStack pose, int x, int y, int mouseX, int mouseY, BigDecimal current_value, BigDecimal max_value) {
		return hovering_power_gauge = renderVerticalGauge(pose, x, y, mouseX, mouseY, level(current_value, max_value), 0, 48, 9, 48);
	}

	protected boolean renderMatterGauge(PoseStack pose, int x, int y, int mouseX, int mouseY, BigDecimal current_value, BigDecimal max_value) {
		return hovering_matter_gauge = renderVerticalGauge(pose, x, y, mouseX, mouseY, level(current_value, max_value), 0, 0, 9, 48);
	}

	protected void renderProgressGauge(PoseStack pose, int x, int y, float level) {
		renderHorizontalGauge(pose, x, y, level, 0, 132, 22, 16);
	}

	protected void renderProgressGaugeLeft(PoseStack pose, int x, int y, float level) {
		renderHorizontalGauge(pose, x - 22, y, level, 0, 132, 22, 16);
	}

	@Override
	public void render(PoseStack p_98418_, int mouseX, int mouseY, float p_98421_) {
		render_x = (width - imageWidth) / 2;
		render_y = (height - imageHeight) / 2;

		for (AbstractWidget<?> widget : menu.widget_list) {
			widget.updateIsHovered(render_x, render_y, mouseX, mouseY);
		}

		this.renderBackground(p_98418_);
		super.render(p_98418_, mouseX, mouseY, p_98421_);
		this.renderTooltip(p_98418_, mouseX, mouseY);
	}

	protected void renderRegularSlot(PoseStack pose, int x, int y) {
		this.blit(pose, render_x + x - 1, render_y + y - 1, 0, 96, 18, 18);
	}

	protected void renderOutputSlot(PoseStack pose, int x, int y) {
		this.blit(pose, render_x + x - 5, render_y + y - 5, 18, 96, 26, 26);
	}

	protected void renderBatterySlot(PoseStack pose, int x, int y) {
		renderRegularSlot(pose, x, y);
		this.blit(pose, render_x + x - 1, render_y + y - 1, 0, 114, 18, 18);
	}

	@Override
	protected void renderBg(PoseStack pose, float p_97788_, int mouseX, int mouseY) {
		RenderSystem.setShader(GameRenderer::getPositionTexShader);
		RenderSystem.setShaderColor(1.0F, 1.0F, 1.0F, 1.0F);
		RenderSystem.setShaderTexture(0, CONTAINER_BACKGROUND());
		// PoseStack, x, y, image_start_x, image_start_y, rect_size_x, rect_size_y
		this.blit(pose, render_x, render_y, 0, 0, this.imageWidth, this.imageHeight);

		renderPowerGauge(pose, mouseX, mouseY);
		renderMatterGauge(pose, mouseX, mouseY);

		RenderSystem.setShader(GameRenderer::getPositionTexShader);
		RenderSystem.setShaderColor(1.0F, 1.0F, 1.0F, 1.0F);
		RenderSystem.setShaderTexture(0, WIDGETS);

		for (Slot slot : menu.slots) {
			if (slot instanceof BatterySlot slot1 && slot1.auto_bg) {
				renderBatterySlot(pose, slot1.x, slot1.y);
			} else if (slot instanceof MachineOutputSlot slot1 && slot1.auto_bg) {
				if (slot1.is_main_output) {
					renderOutputSlot(pose, slot1.x, slot1.y);

					if (progress_supplier != null) {
						renderProgressGaugeLeft(pose, slot1.x - 8, slot1.y, progress_supplier.get());
					}
				} else {
					renderRegularSlot(pose, slot1.x, slot1.y);
				}
			} else if (slot instanceof MachineInputSlot slot1 && slot1.auto_bg) {
				renderRegularSlot(pose, slot1.x, slot1.y);
			}
		}

		for (AbstractWidget widget : menu.widget_list) {
			widget.renderBackground(this, pose, render_x, render_y, mouseX, mouseY);
		}
	}
}
